import os
import shutil

from ocfl_rehydration.drs_descriptor import DrsDescriptor
from ocfl_rehydration.ocfl_inventory import OcflInventory


class Rehydrator():
    """
    A class for rehydrating files from an OCFL object.

    Attributes:
        ocfl_obj_root (str): The path to the root directory of the OCFL object.
        output_dir (str): The path to the directory where rehydrated files will be written.
    """

    def __init__(self, ocfl_obj_root, output_dir) -> None:
        """
        Initializes a Rehydrator object.

        Args:
            ocfl_obj_root (str): The path to the root directory of the OCFL object.
            output_dir (str): The path to the directory where rehydrated files will be written.
        """
        self.ocfl_obj_root = ocfl_obj_root
        self.output_dir = output_dir

    def __repr__(self) -> str:
        return "Rehydrator..."

    def rehydrate(self):
        """
        Rehydrates files from the OCFL object.
        """
        ocfl_inventory = self._get_ocfl_inventory()
        drs_descriptor = self._get_drs_descriptor(ocfl_inventory)
        
        self._rehydrate_files(ocfl_inventory, drs_descriptor)

    def _get_ocfl_inventory(self):
        """
        Gets the OCFL inventory for the OCFL object.

        Returns:
            OcflInventory: The OCFL inventory for the OCFL object.
        """
        inventory_path = os.path.join(self.ocfl_obj_root, 'inventory.json')
        with open(inventory_path, 'r') as file:
            return OcflInventory(file)
        
    def _get_drs_descriptor(self, ocfl_inventory):
        """
        Gets the DRS descriptor for the OCFL object.

        Args:
            ocfl_inventory (OcflInventory): The OCFL inventory for the OCFL object.

        Returns:
            DrsDescriptor: The DRS descriptor for the OCFL object.
        """
        descriptor_path = ocfl_inventory.get_descriptor_path()
        return DrsDescriptor(os.path.join(self.ocfl_obj_root, descriptor_path))
    
    def _rehydrate_files(self, ocfl_inventory, drs_descriptor):
        """
        Rehydrates files from the OCFL object.

        Args:
            ocfl_inventory (OcflInventory): The OCFL inventory for the OCFL object.
            drs_descriptor (DrsDescriptor): The DRS descriptor for the OCFL object.
        """
        output_batch_name = drs_descriptor.get_batch_name()
        output_batch_dir = os.path.join(self.output_dir, output_batch_name)
        for file_id, drs_file in drs_descriptor.files.items():
            data_path = ocfl_inventory.get_data_path(file_id)
            input_data_path = os.path.join(self.ocfl_obj_root, data_path)
            output_dir = os.path.join(output_batch_dir, drs_file.get_file_dir())
            output_file_path = os.path.join(output_dir, drs_file.get_file_name())
            os.makedirs(os.path.dirname(output_file_path), exist_ok=True)
            shutil.copyfile(input_data_path, output_file_path)



import os
import pytest
from project_paths import paths
from ocfl_rehydration.drs_descriptor import DrsDescriptor
from ocfl_rehydration.drs_file import DrsFile


@pytest.fixture
def descriptor_file_1():
    mets_xml = 'data/16952962/v00001/content/descriptor/46296439_mets.xml'
    return os.path.join(paths.dir_unit_resources, mets_xml)

@pytest.fixture
def descriptor_file_2():
    mets_xml = 'data/27340443/v00001/content/descriptor/2573392_mets.xml'
    return os.path.join(paths.dir_unit_resources, mets_xml)

@pytest.fixture
def expected_1():
    drs_files = {}
    f1 = DrsFile()
    f1.set_id('46296441')
    f1.set_file_name('vzocr2.TXT')
    f1.set_file_dir('ocr_uncorrected')
    drs_files[f1.get_id()] = f1

    f2 = DrsFile()
    f2.set_id('46296442')
    f2.set_file_name('vzocr3.TXT')
    f2.set_file_dir('ocr_uncorrected')
    drs_files[f2.get_id()] = f2

    f3 = DrsFile()
    f3.set_id('46296440')
    f3.set_file_name('vzocr1.txt')
    f3.set_file_dir('ocr_uncorrected')
    drs_files[f3.get_id()] = f3

    return drs_files

@pytest.fixture
def expected_2():
    drs_files = {}
    f1 = DrsFile()
    f1.set_id('1028168')
    f1.set_file_name('_005705781_0001.txt')
    f1.set_file_dir('/')
    f1.set_digest_alg('MD5')
    f1.set_digest_value('bb6df4cc25596c6dd8c6cb9a8060cf40')
    f1.set_mime_type('text/plain')
    drs_files[f1.get_id()] = f1

    f2 = DrsFile()
    f2.set_id('1028170')
    f2.set_file_name('_005705781_0002.txt')
    f2.set_file_dir('/')
    f2.set_digest_alg('MD5')
    f2.set_digest_value('e001ee7dc7134dcda7192486103673d5')
    f2.set_mime_type('text/plain')
    drs_files[f2.get_id()] = f2

    f3 = DrsFile()
    f3.set_id('1028172')
    f3.set_file_name('_005705781_0003.txt')
    f3.set_file_dir('/')
    f3.set_digest_alg('MD5')
    f3.set_digest_value('e9d96ff68f4bb21a65d9206157058f0e')
    f3.set_mime_type('text/plain')
    drs_files[f3.get_id()] = f3

    f4 = DrsFile()
    f4.set_id('1028174')
    f4.set_file_name('_005705781_0004.txt')
    f4.set_file_dir('/')
    f4.set_digest_alg('MD5')
    f4.set_digest_value('4154d62053636345905fcc91dbb9912a')
    f4.set_mime_type('text/plain')
    drs_files[f4.get_id()] = f4

    f5 = DrsFile()
    f5.set_id('1028169')
    f5.set_file_name('_005705781_0002.tif')
    f5.set_file_dir('/')
    f5.set_digest_alg('MD5')
    f5.set_digest_value('9e1cd75876dddc251d2385048bdce23a')
    f5.set_mime_type('image/tiff')
    drs_files[f5.get_id()] = f5

    f6 = DrsFile()
    f6.set_id('1028173')
    f6.set_file_name('_005705781_0004.tif')
    f6.set_file_dir('/')
    f6.set_digest_alg('MD5')
    f6.set_digest_value('16c958906cd869926c5c0f04b24f94c5')
    f6.set_mime_type('image/tiff')
    drs_files[f6.get_id()] = f6

    f7 = DrsFile()
    f7.set_id('1028167')
    f7.set_file_name('_005705781_0001.tif')
    f7.set_file_dir('/')
    f7.set_digest_alg('MD5')
    f7.set_digest_value('4ddc12699510696b7428e31fce489868')
    f7.set_mime_type('image/tiff')
    drs_files[f7.get_id()] = f7

    f8 = DrsFile()
    f8.set_id('1028171')
    f8.set_file_name('_005705781_0003.tif')
    f8.set_file_dir('/')
    f8.set_digest_alg('MD5')
    f8.set_digest_value('d3bc8e8feb3473f97a53cce87a5a1804')
    f8.set_mime_type('image/tiff')
    drs_files[f8.get_id()] = f8

    return drs_files


def test_parse_1(descriptor_file_1, expected_1):
    descriptor = DrsDescriptor(descriptor_file_1)

    # Verify files
    files = descriptor.get_files()
    assert len(files) == len(expected_1)

    for f in files.values():
        expected_file = expected_1.get(f.get_id())
        assert expected_file is not None
        assert f.get_id() == expected_file.get_id()
        assert f.get_file_name() == expected_file.get_file_name()
        assert f.get_file_dir() == expected_file.get_file_dir()
        expected_1.pop(f.get_id())

    assert len(expected_1) == 0


def test_parse_2(descriptor_file_2, expected_2):
    descriptor = DrsDescriptor(descriptor_file_2)

    # Verify files
    files = descriptor.get_files()
    assert len(files) == len(expected_2)

    for f in files.values():        
        expected_file = expected_2.get(f.get_id())
        assert expected_file is not None
        assert f.get_id() == expected_file.get_id()
        assert f.get_file_name() == expected_file.get_file_name()
        assert f.get_file_dir() == expected_file.get_file_dir()
        assert f.get_digest_alg() == expected_file.get_digest_alg()
        assert f.get_digest_value() == expected_file.get_digest_value()
        assert f.get_mime_type() == expected_file.get_mime_type()
        expected_2.pop(f.get_id())

    assert len(expected_2) == 0


def test_get_batch_name():
    descriptor_file_1 = 'data/27997603/v00001/content/descriptor/2573685_mets.xml'
    descriptor_path_1 = os.path.join(paths.dir_unit_resources, descriptor_file_1)
    descriptor_1 = DrsDescriptor(descriptor_path_1)
    assert descriptor_1.get_batch_name() == '1339763.xml'

    descriptor_file_2 = 'data/27340448/v00001/content/descriptor/2573397_mets.xml'
    descriptor_path_2 = os.path.join(paths.dir_unit_resources, descriptor_file_2)
    descriptor_2 = DrsDescriptor(descriptor_path_2)
    assert descriptor_2.get_batch_name() == '1028626.xml'


import argparse
import os
from ocfl_inventory import OcflInventory
from rehydrator import Rehydrator

###
## Main ##
###
if __name__ == '__main__':
  # construct the argument parse and parse the arguments
  ap = argparse.ArgumentParser(description=
                  "Converts the OCFL form of a DRS Object and reconstitutes "\
                  "(rehydrates) a form expected by curators. "\
                  "The input is the OCFL object root directory of object to "\
                  "rehydrate.")

  ap.add_argument('-i', '--input_dir',
                  required=True,
                  help='Local directory containing the OCFL Object root of '\
                       'the object to rehydrate')
  ap.add_argument('-o', '--output_dir',
                  required=True,
                  help='Local directory where rehydrated object will be written')
  args = vars(ap.parse_args())
 
  input_dir  = args['input_dir']
  output_dir = args['output_dir']

  rehydrator = Rehydrator(input_dir, output_dir)
  rehydrator.rehydrate()


import os
import pytest
from project_paths import paths
from ocfl_rehydration.ocfl_inventory import OcflInventory


@pytest.fixture
def inventory_file():
    inventory_json = 'data/16952962/v00001/inventory.json'
    return os.path.join(paths.dir_unit_resources, inventory_json)


def test_get_descriptor_path(inventory_file):
    inventory = OcflInventory(open(inventory_file))
    descriptor_path = inventory.get_descriptor_path()
    assert descriptor_path == 'v00001/content/descriptor/46296439_mets.xml'


def test_get_data_path(inventory_file):
    inventory = OcflInventory(open(inventory_file))
    data_path = inventory.get_data_path('46296441')
    assert data_path == 'v00001/content/data/46296441.txt'


class DrsFile:
    """
    A class representing a DRS file.
    """

    def __init__(self):
        """
        Constructor.
        """
        self.file_id = ""
        self.file_name = ""
        self.file_dir = ""
        self.digest_alg = ""
        self.digest_value = ""
        self.mime_type = ""
        self.amd_ids = []

    def __repr__(self):
        """
        Returns a string representation of this object.
        """
        return "DrsFile: id={}, file_name={}, file_dir={}, digest_alg={}, "\
               "digest_value={}, mime_type={}".format(self.file_id,
                                                      self.file_name,
                                                      self.file_dir,
                                                      self.digest_alg,
                                                      self.digest_value,
                                                      self.mime_type)

    def get_id(self):
        """
        Returns the id of this file.

        Returns:
            int: The id of this file.
        """
        return self.file_id


    def get_file_name(self):
        """
        Returns the supplied name of the file.

        Returns:
            str: The name of the file.
        """
        return self.file_name
    
    
    def get_file_dir(self):
        """
        Returns the supplied directory containing the file.

        Returns:
            str: The directory containing the file.
        """
        if self.file_dir == '/':
            return ''
        return self.file_dir
    
    def add_amd_id(self, amd_id):
        """
        Adds an amd_id to the list of amd_ids for this file.

        Args:
            amd_id (str): The amd_id to add.
        """
        self.amd_ids.append(amd_id)

    def get_digest_alg(self):
        """
        Returns the digest algorithm used for this file.

        Returns:
            str: The digest algorithm used for this file.
        """
        return self.digest_alg

    def get_digest_value(self):
        """
        Returns the digest value for this file.

        Returns:
            str: The digest value for this file.
        """
        return self.digest_value

    def get_mime_type(self):
        """
        Returns the mime type for this file.

        Returns:
            str: The mime type for this file.
        """
        return self.mime_type

    def set_id(self, file_id):
        """
        Sets the id of this file.

        Args:
            file_id (int): The id of this file.
        """
        self.file_id = file_id

    def set_file_name(self, file_name):
        """
        Sets the supplied name of the file.

        Args:
            file_name (str): The name of the file.
        """
        self.file_name = file_name

    def set_file_dir(self, file_dir):
        """
        Sets the supplied directory containing the file.

        Args:
            file_dir (str): The directory containing the file.
        """
        self.file_dir = file_dir

    def set_digest_alg(self, digest_alg):
        """
        Sets the digest algorithm used for this file.

        Args:
            digest_alg (str): The digest algorithm used for this file.
        """
        self.digest_alg = digest_alg

    def set_digest_value(self, digest_value):
        """
        Sets the digest value for this file.

        Args:
            digest_value (str): The digest value for this file.
        """
        self.digest_value = digest_value

    def set_mime_type(self, mime_type):
        """
        Sets the mime type for this file.

        Args:
            mime_type (str): The mime type for this file.
        """
        self.mime_type = mime_type


import re
import json


import json
import re

class OcflInventory():
    """
    A class representing an OCFL inventory.

    Attributes:
    - inventory (dict): The inventory represented as a dictionary.
    """

    def __init__(self, file):
        """
        Initializes an OcflInventory object.

        Args:
        - file (file): The file object containing the inventory data.
        """
        self.inventory = json.load(file)
        

    def get_descriptor_path(self):
        """
        Returns the path to the descriptor file for the object.

        Returns:
        - str: The path to the descriptor file.

        Raises:
        - Exception: If the descriptor file is not found.
        """
        path_regex = r'^v[0-9]{5}/content/descriptor/.*_mets.xml'
        manifest_files = self._get_manifest_files(path_regex)

        if len(manifest_files) > 0:
            return sorted(manifest_files, reverse=True)[0]

        raise Exception("not-found, descriptor")


    def get_data_path(self, file_id):
        """
        Returns the path to the data file for the given file ID.

        Args:
        - file_id (str): The ID of the file.

        Returns:
        - str: The path to the data file.

        Raises:
        - Exception: If the data file is not found.
        """
        path_regex = r'^data/' + file_id + r'.*'
        digest = self._get_state_files_digest(path_regex=path_regex, version=None)
        return self._get_manifest_path(digest)


    def _get_state_files_digest(self, path_regex, version):
        if version is None:
            version = self.inventory['head']

        state = self.inventory['versions'][version]['state']

        if not isinstance(state, dict):
            raise Exception('Invalid state')

        for digest in state:
            for file in state[digest]:
                if (re.search(path_regex, file)):
                    return digest

        raise Exception("not-found, state file digest for: {} : {}".format(version, path_regex))

    def _get_manifest_path(self, digest):
        manifest = self.inventory['manifest']

        if not isinstance(manifest, dict):
            raise Exception('Invalid manifest')

        for file in manifest[digest]:
            return file

        raise Exception("not-found, manifest file for digest: {}".format(digest))

    def _get_manifest_files(self, path_regex):
        """
        Returns a set of file paths that match the given regular expression.

        Args:
        - path_regex (str): The regular expression to match.

        Returns:
        - set: A set of file paths that match the regular expression.

        Raises:
        - Exception: If the manifest is invalid.
        """
        manifest = self.inventory['manifest']

        if not isinstance(manifest, dict):
            raise Exception('Invalid manifest')
    
        manifest_files = set() 
        for digest in manifest:
            for file in manifest[digest]:
                if (re.search(path_regex, file)):
                    manifest_files.add(file)

        return manifest_files



from lxml import etree
from .drs_file import DrsFile
import logging

class DrsDescriptor():
  
  NSMAP = {
    "mets" : "{http://www.loc.gov/METS/}",
    "hulDrsAdmin" : "{http://hul.harvard.edu/ois/xml/ns/hulDrsAdmin}",
    "premis" : "{info:lc/xmlns/premis-v2}",
    "xsi" : "{http://www.w3.org/2001/XMLSchema-instance}",
    "fits" : "{http://hul.harvard.edu/ois/xml/ns/fits/fits_output}"
  }
  
  def __init__(self, descriptor_file):
    self.files = {}
    self.root = etree.parse(descriptor_file).getroot()
    self._parse()

  def __repr__(self) -> str:
    out = ""
    if len(self.files) > 0:
      for f in self.files:
        out += f'{f}\n'
    else:
      out += 'No files found\n'

    if len(self.amds) > 0:
      for amd in self.amds:
        out += f'{amd}\n'
    else:  
      out += 'No AMDs found\n'

    return out


  def _parse(self):
    # Collect METS:files and their corresponding METS:AmdSecs
    file_grps = self.root.findall('.//{}fileGrp'.format(self.NSMAP['mets']))
    grps = {}
    for file_grp in file_grps:
      file_grp_id = file_grp.get('ID')
      grp = {file_grp_id: {}}

      files = file_grp.findall('{}file'.format(self.NSMAP['mets']))
      for file in files:
        file_id = file.get('ID')
        file_amd_ids = file.get('ADMID')      
        grp[file_grp_id][file_id] = file_amd_ids

      grps.update(grp)
        
    # Associate METS:files with DRS:files
    for grp in grps.values():
      for file_id, amd_ids in grp.items():
        drs_file = self._create_drs_file(amd_ids)
        self.files[drs_file.get_id()] = drs_file

  
  def _create_drs_file(self, amd_ids):
    """
    Given a list of amd_ids, create a DRS file object.
    """
    drs_file = DrsFile()
    amd_secs = self.root.findall(".//{}amdSec".format(self.NSMAP['mets']))

    for amd_id in amd_ids.split():

      amd_premis_file = self._amdSec_premis_file(amd_secs, amd_id)
      if amd_premis_file is not None:
        drs_file.set_id(amd_premis_file.find('.//{}objectIdentifierValue'.format(self.NSMAP['premis'])).text)
        drs_file.set_digest_alg(amd_premis_file.find('.//{}messageDigestAlgorithm'.format(self.NSMAP['premis'])).text)
        drs_file.set_digest_value(amd_premis_file.find('.//{}messageDigest'.format(self.NSMAP['premis'])).text)
        continue

      amd_drs_file = self._amdSec_drs_file(amd_secs, amd_id)
      if amd_drs_file is not None:
        drs_file.set_file_name(amd_drs_file.find('.//{}suppliedFilename'.format(self.NSMAP['hulDrsAdmin'])).text)
        drs_file.set_file_dir(amd_drs_file.find('.//{}suppliedDirectory'.format(self.NSMAP['hulDrsAdmin'])).text)
        drs_file.set_mime_type(amd_drs_file.find('.//{}identity'.format(self.NSMAP['fits'])).get('mimetype'))
        continue

    return drs_file 

    
  def _amdSec_premis_file(self, amd_secs, amd_id):    
    for amd_sec in amd_secs:
        if amd_sec.get('ID') == amd_id:
           premis_obj = amd_sec.find('.//{}object'.format(self.NSMAP['premis']))
           if premis_obj is not None and premis_obj.get('{}type'.format(self.NSMAP['xsi'])) == 'premis:file':
              return premis_obj
    return None
  
    
  def _amdSec_drs_file(self, amd_secs, amd_id):
    for amd_sec in amd_secs:
        if amd_sec.get('ID') == amd_id:
           amd_drs_file = amd_sec.find('.//{}drsFile'.format(self.NSMAP['hulDrsAdmin']))
           if amd_drs_file is not None:
              return amd_drs_file
    return None    
  

  def _amdSec_premis_representation(self, amd_secs, amd_id):    
    for amd_sec in amd_secs:
        if amd_sec.get('ID') == amd_id:
           premis_obj = amd_sec.find('.//{}object'.format(self.NSMAP['premis']))
           if premis_obj is not None and premis_obj.get('{}type'.format(self.NSMAP['xsi'])) == 'premis:representation':
              return premis_obj
    return None
  

  def get_batch_name(self):
    mets_hdr = self.root.find("{}metsHdr".format(self.NSMAP['mets']))
    mets_hdr_amd_ids = mets_hdr.get('ADMID')      
    amd_secs = self.root.findall(".//{}amdSec".format(self.NSMAP['mets']))

    for amd_id in mets_hdr_amd_ids.split():
      amd_premis_representation = self._amdSec_premis_representation(amd_secs, amd_id)
      if amd_premis_representation is not None:
        return amd_premis_representation.find('.//{}originalName'.format(self.NSMAP['premis'])).text


  def get_files(self):
    return self.files





import setuptools
setuptools.setup(name='ocfl_rehydration',
version='0.2',
description='Tooling for rehydrating an original DRS object from its OCFL form',
url='https://github.com/harvard-lts/ocfl_rehydration',
author='Andrew Woods',
install_requires=[''],
author_email='andrew_woods@harvard.edu',
packages=['ocfl_rehydration'], 
zip_safe=False)



import os
import shutil
import pytest
from project_paths import paths
from ocfl_rehydration.rehydrator import Rehydrator

@pytest.fixture
def batch_name():
    return "1028626.xml"

@pytest.fixture
def output_dir():
    return os.path.join(paths.dir_unit_out, 'rehydrator_test')

@pytest.fixture
def rehydrator(output_dir):
    ocfl_object_path = os.path.join(paths.dir_unit_resources, 'data/27340448')
    return Rehydrator(ocfl_object_path, output_dir)

@pytest.fixture
def expected_files():
    return [
        "_002635021_0001.tif",
        "_002635021_0001.txt",
        "_002635021_0002.tif",
        "_002635021_0002.txt",
        "_002635021_0003.tif",
        "_002635021_0003.txt",
        "_002635021_0004.tif",
        "_002635021_0004.txt",
        "_002635021_0005.tif",
        "_002635021_0005.txt",
        "_002635021_0006.tif",
        "_002635021_0006.txt"
    ]


def test_rehydrate(rehydrator, expected_files, output_dir, batch_name):
    rehydrator.rehydrate()

    for expected_file in expected_files:
        assert os.path.exists(os.path.join(output_dir, batch_name, expected_file))

    shutil.rmtree(output_dir)